#!/usr/bin/python

import actions
import yaml
import os
import random

from charmhelpers.core import services

from charmhelpers.core.hookenv import (
    config,
    unit_get,
)


def cooked_config():
    conf = config()
    conf['yaml'] = {}

    unit = os.environ.get('JUJU_UNIT_NAME').split('/')[-1]
    conf['yaml']['servername'] = "{}.{}".format(unit, conf['servername'])
    # ssl_ports requires both ssl_key and ssl_certificiate
    if not conf.get('ssl_key') or not conf.get('ssl_certificate'):
        conf['ssl_ports'] = '[]'

    # cook the various yaml formatted strings into their true form
    for key in 'opertype_list', 'oper_list':
        val = yaml.safe_load(conf[key])
        if not val:
            val = []
        conf['yaml'][key] = val
    val = yaml.safe_load(conf['ports'])
    if not val:
        val = [['0.0.0.0', 0]]
    conf['yaml']['ports'] = val
    # we overload 'yaml' with a touch of non-yaml, cooked data
    conf['yaml']['modules'] = conf['modules'].split()
    conf['seed'] = int(random.random() * 10000000)
    return conf


class IRCdServicesRelation(services.RelationContext):
    name = 'ircdservices'
    inteface = 'services'

    # services framework gets lost if we actually say required_keys.
    # required_keys = ['host', 'port', 'ssl', 'password', 'protocol', 'networkname']

    def provide_data(self):
        ret = {}
        conf = config()
        ret['servername'] = conf['servername']
        ret['password'] = conf['password']
        ret['host'] = unit_get('private-address')
        ret['protocol'] = 'anope'
        return ret


class MysqlRelation(services.RelationContext):
    name = 'db'
    interface = 'mysql'
    required_keys = ['host', 'user', 'password', 'database']

    def service_available(service_name):
        # not a real service
        return False


service_definitions = [
    {
        'service': 'anope',
        'ports': [],
        'required_data': [cooked_config(), IRCdServicesRelation(), MysqlRelation()],
        'data_ready': [
            actions.install_package,
            services.render_template(source="default.tmpl", target="/etc/default/anope"),
            services.render_template(source="apparmor.tmpl", target="/etc/apparmor.d/usr.sbin.anope"),
            services.render_template(source="nagios.tmpl", target="/usr/lib/nagios/plugins/check_anope", perms=0755),

            services.render_template(source="services.conf.tmpl", target="/etc/anope/services.conf"),
            services.render_template(source="botserv.conf.tmpl", target="/etc/anope/botserv.conf"),
            services.render_template(source="chanserv.conf.tmpl", target="/etc/anope/chanserv.conf"),
            services.render_template(source="global.conf.tmpl", target="/etc/anope/global.conf"),
            services.render_template(source="hostserv.conf.tmpl", target="/etc/anope/hostserv.conf"),
            services.render_template(source="irc2sql.conf.tmpl", target="/etc/anope/irc2sql.conf"),
            services.render_template(source="memoserv.conf.tmpl", target="/etc/anope/memoserv.conf"),
            services.render_template(source="nickserv.conf.tmpl", target="/etc/anope/nickserv.conf"),
            services.render_template(source="operserv.conf.tmpl", target="/etc/anope/operserv.conf"),
            actions.enable_apparmor_profile,
        ],
        'provided_data': [IRCdServicesRelation()],
        'start': [actions.anope_reload, services.open_ports],
    },
]
